import pandas as pd
import numpy as np   

def aggr(x1:pd.DataFrame,
         x2:pd.DataFrame, verbose:bool=False) -> pd.DataFrame:
    """ Input 2 dataframes with exactly 1 column, which is indexed."""
    """ Note : t(x1) << t(x2)"""
    #Find common
    common = pd.merge(x1, x2, left_index=True, right_index=True)
    print("Potential Ratio Points : ", len(common)) if verbose else None
    #Drop Inf and 0
    common = common.replace([np.inf, -np.inf, 0], np.nan).dropna()
    print("Usable Ratio Points : ", len(common)) if verbose else None
    if len(common) < 1:
        raise ValueError("Ratio could not be calculated, try bigger overlap.")
    #Calc Ratio
    common["ratio"] = common[common.columns[0]] / common[common.columns[1]]
    ratio = common["ratio"].value_counts().idxmax()
    df = x1.append(x2.drop(pd.merge(x1, x2, left_index=True, right_index=True).index)*ratio)
    return df/max(df[df.columns[0]])*100 if ratio > 1 else df

def qAggr(frames:list, verbose:bool=False) -> pd.DataFrame:
    """convienent function to quick-aggregate n data pieces"""
    """IMPORTANT : input needs to be ordered where t(args[0]) < t(args[1]) < ... < t(args[-1])"""
    df = aggr(frames[0], frames[1], verbose=verbose)
    for x in frames[2:]:
        df = aggr(df, x, verbose=verbose)
    return df.asfreq("D")



import pandas as pd
import numpy as np
from functools import reduce

from DailyTrends.request import collect_frames
from DailyTrends.rescale import qAggr

def collect_data(q:str,
                 save:bool=False,
                dest:str=None, verbose:bool=False) -> pd.DataFrame:
  if type(q) == str:
    #gets the unscaled data pieces
    frames = collect_frames(q)
    #"quick-aggregates" the data pieces and rescales the values using overlaps
    df = qAggr(frames, verbose=verbose)
    return df if not save else df.to_csv(dest)
  elif type(q) == list:
    return reduce(lambda left,right: pd.merge(left,right,left_index=True, right_index=True),
                  [collect_data(i, save, dest) for i in q])
  else:
    raise TypeError("Use string for single query search / list of strings for multiple keywoards.")
    
    



import io
from datetime import datetime
from dateutil.relativedelta import relativedelta as delta
import requests
import urllib
import json
import random
import pandas as pd
import numpy as np
import time
from tqdm import tqdm

def generate_intervals(overlap:int=40,
                       inc:int=265,
                      start:str="2004-01-01") -> list:
    """ 
    start defaults to "2004-01-01", which represents the entire series.
    Format : "YYYY-MM-DD"  
    """
    to_str = lambda dt_date : datetime.strftime(dt_date, "%Y-%m-%d")
    to_dt = lambda str_date : datetime.strptime(str_date, "%Y-%m-%d")
    n_iter = 25
    intervals = []
    for i in range(n_iter):
        start = "2004-01-01" if i == 0 else to_str(to_dt(end) + delta(days=-overlap))
        end = to_str(to_dt(start) + delta(days=+inc))
        intervals.append(start + " " + end)  
    return intervals

def get_frame(q:str, time:str) -> pd.DataFrame:
    jar = requests.get("https://trends.google.com/").cookies
    opener = urllib.request.build_opener(urllib.request.HTTPCookieProcessor(jar))
    opener.addheaders = [
    			("Referrer", "https://trends.google.com/trends/explore"),
    			('User-Agent', 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.21 (KHTML, like Gecko) Chrome/19.0.1042.0 Safari/535.21'),
    			("Accept", "text/plain") ]
    params_1 = None
    params_0 = {
                            "hl": "en-US",
                            "tz": -120,
                            "req": {
                            "comparisonItem": [
                                    {
                                            "keyword": q,
                                            "geo":"",
                                            "time": time
                                    }
                                    ],
                                    "category": 0,
                                    "property": "" }}
    params_0["req"] = json.dumps(params_0["req"], separators=(',', ':')) 
    params_0 = urllib.parse.urlencode(params_0)
    params_0 = params_0.replace('%3A', ':').replace('%2C', ',')  
                   
    data = opener.open("https://trends.google.com/trends/api/explore?" + params_0).read().decode('utf8')
    data = data[data.find("{"):]
    data = json.loads(data)
    widgets = data["widgets"]
    
    
    for widget in widgets:
        if widget["title"] == "Interest over time":
            params_1 = {
                                            "req":widget["request"],
                                            "token":widget["token"],
                                            "tz":-120
                                    }
    params_1["req"] = json.dumps(params_1["req"],separators=(',', ':'))
    params_1 = urllib.parse.urlencode(params_1).replace("+", "%20")
    csv_url = 'https://trends.google.com/trends/api/widgetdata/multiline/csv?' + params_1
    result = opener.open(csv_url).read().decode('utf8')
    return pd.read_csv(io.StringIO(result), skiprows=range(0,1), index_col=0, header=0).asfreq("d")

def collect_frames(q:str) -> list:
    intervals = generate_intervals()
    frames = []
    for interval in tqdm(intervals):
        df = get_frame(q, interval)
        time.sleep(random.gammavariate(.99,1.99))
        if len(df) == 0:
            continue
        frames.append(df)
    return frames






from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = ["pandas>=0.24.2", "numpy>=1.16.4", "requests>=2.22.0", "tqdm>=4.35.0"]

setup(
    name="DailyTrends",
    version="1.0",
    author="Leonard Vorbeck",
    author_email="leomxyy@googlemail.com",
    description="A package to receive full-scale daily Google Trends data",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/le0x99/DailyTrends/",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)



import random
import pandas as pd
import numpy as np

from DailyTrends.rescale import aggr, qAggr


### Create toy dfs to check consistency of the rescaler

# draws random numbers with a weight on zeros, since they occure pretty often in the trends data
def drawer() -> int:
    return random.choice([0] * 100 + list(np.random.randint(0, 10_000,
                         dtype="int",
                         size=1000))  )
 #toy df   
df = pd.DataFrame({"x" : [ drawer() for _ in range(284)]},
                          index = pd.date_range(start="1/1/2004", end="10/10/2004", freq="d"))

#MaxScale the frames
def MaxScale(_:pd.DataFrame or pd.Series ) -> pd.DataFrame:
    return (_/_.values.max() * 100) if type(_) == pd.Series else pd.DataFrame( { col : MaxScale(_[col]) for col in _.columns })

scaled_df = MaxScale(df)

# Slice the df into 4 overlapping pieces and scale them into [0, 100]
frac = int(len(df)/4)
ol = 20 #overlap
a, b = MaxScale(df[:frac+ol]), MaxScale(df[frac:frac*2+ol])
c, d = MaxScale(df[frac*2: frac*3+ol]), MaxScale(df[frac*3:])

rescaled_df = qAggr([a, b, c, d])

check = pd.DataFrame({"true" : df.x.values, 
              "scaled" : scaled_df.x.values,
              "rescaled" : rescaled_df.x.values })



